@@ -1,6 +1,6 @@
 /*
  *  Phusion Passenger - https://www.phusionpassenger.com/
- *  Copyright (c) 2010-2017 Phusion Holding B.V.
+ *  Copyright (c) 2010-2018 Phusion Holding B.V.
  *
  *  "Passenger", "Phusion Passenger" and "Union Station" are registered
  *  trademarks of Phusion Holding B.V.
@@ -30,14 +30,13 @@
 #include <vector>
 #include <utility>
 #include <boost/shared_array.hpp>
-#include <AppTypes.h>
+#include <WrapperRegistry/Registry.h>
 #include <DataStructures/HashedStaticString.h>
 #include <Constants.h>
 #include <ResourceLocator.h>
 #include <StaticString.h>
+#include <FileTools/PathManip.h>
 #include <Utils.h>
-#include <Core/UnionStation/Context.h>
-#include <Core/UnionStation/Transaction.h>
 
 namespace Passenger {
 namespace ApplicationPool2 {
@@ -84,8 +83,9 @@ private:
 
 		result.push_back(&options.appRoot);
 		result.push_back(&options.appGroupName);
+		result.push_back(&options.appLogFile);
 		result.push_back(&options.appType);
-		result.push_back(&options.startCommand);
+		result.push_back(&options.appStartCommand);
 		result.push_back(&options.startupFile);
 		result.push_back(&options.processTitle);
 
@@ -110,13 +110,10 @@ private:
 		result.push_back(&options.meteorAppSettings);
 
 		result.push_back(&options.environmentVariables);
-		result.push_back(&options.ustRouterAddress);
-		result.push_back(&options.ustRouterUsername);
-		result.push_back(&options.ustRouterPassword);
 		result.push_back(&options.apiKey);
+		result.push_back(&options.groupUuid);
 		result.push_back(&options.hostName);
 		result.push_back(&options.uri);
-		result.push_back(&options.unionStationKey);
 
 		return result;
 	}
@@ -176,28 +173,31 @@ public:
 	 */
 	HashedStaticString appGroupName;
 
+	/** The application's log file, where Passenger sends the logs from
+	 *	the application.
+	 */
+	StaticString appLogFile;
+
 	/** The application's type, used for determining the command to invoke to
 	 * spawn an application process as well as determining the startup file's
 	 * filename. It can be one of the app type names in AppType.cpp, or the
-	 * empty string (default). In case of the latter, 'startCommand' and
+	 * empty string (default). In case of the latter, 'appStartCommand' and
 	 * 'startupFile' (which MUST be set) will dictate the startup command
 	 * and the startup file's filename. */
 	StaticString appType;
 
-	/** The command for spawning the application process. This is a list of
-	 * arguments, separated by '\t', e.g. "ruby\tfoo.rb". Only used
-	 * during spawning and only if appType.empty(). */
-	StaticString startCommand;
+	/** The shell command string for spawning the application process.
+	 * Only used during spawning and only if appType.empty().
+	 */
+	StaticString appStartCommand;
 
 	/** Filename of the application's startup file. Only actually used for
-	 * determining user switching info. Only used during spawning and only
-	 * if appType.empty(). */
+	 * determining user switching info. Only used during spawning. */
 	StaticString startupFile;
 
 	/** The process title to assign to the application process. Only used
 	 * during spawning. May be empty in which case no particular process
-	 * title is assigned. Only used during spawning and only if
-	 * appType.empty(). */
+	 * title is assigned. Only used during spawning. */
 	StaticString processTitle;
 
 	/**
@@ -305,23 +305,6 @@ public:
 
 	bool userSwitching;
 
-	/** Whether Union Station logging should be enabled. Enabling this option will
-	 * result in:
-	 *
-	 *  - The application enabling its Union Station support.
-	 *  - Periodic tasks such as `collectAnalytics()` to log things to Union Station.
-	 *
-	 * It does *not* necessarily result in a request logging data to Union Station.
-	 * That depends on whether the `transaction` member is set.
-	 *
-	 * If this is set to true, then 'ustRouterAddress', 'ustRouterUsername'
-	 * and 'ustRouterPassword' must be non-empty.
-	 */
-	bool analytics;
-	StaticString ustRouterAddress;
-	StaticString ustRouterUsername;
-	StaticString ustRouterPassword;
-
 	/**
 	 * Whether Spawner should raise an internal error when spawning. Used
 	 * during unit tests.
@@ -370,19 +353,6 @@ public:
 	 */
 	bool abortWebsocketsOnProcessShutdown;
 
-	/**
-	 * The Union Station key to use in case analytics logging is enabled.
-	 * It is used by Pool::collectAnalytics() and other administrative
-	 * functions which are called periodically. Because they do not belong
-	 * to any request, and they may still want to log to Union Station,
-	 * this key is stored in the per-group options structure.
-	 *
-	 * It is not used on a per-request basis. Per-request analytics logging
-	 * (and Union Station logging) uses the logger object in the `logger` field
-	 * instead.
-	 */
-	StaticString unionStationKey;
-
 	/*-----------------*/
 
 
@@ -398,18 +368,6 @@ public:
 	/** Current request URI. */
 	StaticString uri;
 
-	/**
-	 * The Union Station log transaction that this request belongs to.
-	 * May be the null pointer, in which case Union Station logging is
-	 * disabled for this request.
-	 *
-	 * When an Options object is passed to another thread (either direct or through
-	 * a copy), the caller should call `detachFromUnionStationTransaction()`.
-	 * Each Union Station transaction object is only supposed to be used in the same
-	 * thread.
-	 */
-	UnionStation::TransactionPtr transaction;
-
 	/**
 	 * A sticky session ID for routing to a specific process.
 	 */
@@ -472,14 +430,14 @@ public:
 	 */
 	Options()
 		: logLevel(DEFAULT_LOG_LEVEL),
-		  startTimeout(90 * 1000),
+		  startTimeout(DEFAULT_START_TIMEOUT),
 		  environment(DEFAULT_APP_ENV, sizeof(DEFAULT_APP_ENV) - 1),
 		  baseURI("/", 1),
 		  spawnMethod(DEFAULT_SPAWN_METHOD, sizeof(DEFAULT_SPAWN_METHOD) - 1),
 		  defaultUser(PASSENGER_DEFAULT_USER, sizeof(PASSENGER_DEFAULT_USER) - 1),
 		  lveMinUid(DEFAULT_LVE_MIN_UID),
 		  integrationMode(DEFAULT_INTEGRATION_MODE, sizeof(DEFAULT_INTEGRATION_MODE) - 1),
-		  ruby(DEFAULT_RUBY, sizeof(DEFAULT_RUBY) - 1),
+		  ruby("/usr/local/bin/ruby26", sizeof("/usr/local/bin/ruby26") - 1),
 		  python(DEFAULT_PYTHON, sizeof(DEFAULT_PYTHON) - 1),
 		  nodejs(DEFAULT_NODEJS, sizeof(DEFAULT_NODEJS) - 1),
 		  fileDescriptorUlimit(0),
@@ -487,14 +445,13 @@ public:
 		  debugger(false),
 		  loadShellEnvvars(true),
 		  userSwitching(true),
-		  analytics(false),
 		  raiseInternalError(false),
 
 		  minProcesses(1),
 		  maxProcesses(0),
 		  maxPreloaderIdleTime(-1),
 		  maxOutOfBandWorkInstances(1),
-		  maxRequestQueueSize(100),
+		  maxRequestQueueSize(DEFAULT_MAX_REQUEST_QUEUE_SIZE),
 		  abortWebsocketsOnProcessShutdown(true),
 
 		  stickySessionId(0),
@@ -573,11 +530,6 @@ public:
 		stickySessionId = 0;
 		currentTime     = 0;
 		noop     = false;
-		return detachFromUnionStationTransaction();
-	}
-
-	Options &detachFromUnionStationTransaction() {
-		transaction.reset();
 		return *this;
 	}
 
@@ -593,15 +545,16 @@ public:
 	 * the `elements` argument.
 	 */
 	void toVector(vector<string> &vec, const ResourceLocator &resourceLocator,
-		int fields = ALL_OPTIONS) const
+		const WrapperRegistry::Registry &wrapperRegistry, int fields = ALL_OPTIONS) const
 	{
 		if (fields & SPAWN_OPTIONS) {
 			appendKeyValue (vec, "app_root",           appRoot);
 			appendKeyValue (vec, "app_group_name",     getAppGroupName());
 			appendKeyValue (vec, "app_type",           appType);
-			appendKeyValue (vec, "start_command",      getStartCommand(resourceLocator));
-			appendKeyValue (vec, "startup_file",       absolutizePath(getStartupFile(), absolutizePath(appRoot)));
-			appendKeyValue (vec, "process_title",      getProcessTitle());
+			appendKeyValue (vec, "app_log_file",       appLogFile);
+			appendKeyValue (vec, "start_command",      getStartCommand(resourceLocator, wrapperRegistry));
+			appendKeyValue (vec, "startup_file",       absolutizePath(getStartupFile(wrapperRegistry), absolutizePath(appRoot)));
+			appendKeyValue (vec, "process_title",      getProcessTitle(wrapperRegistry));
 			appendKeyValue2(vec, "log_level",          logLevel);
 			appendKeyValue3(vec, "start_timeout",      startTimeout);
 			appendKeyValue (vec, "environment",        environment);
@@ -619,11 +572,7 @@ public:
 			appendKeyValue (vec, "python",             python);
 			appendKeyValue (vec, "nodejs",             nodejs);
 			appendKeyValue (vec, "meteor_app_settings", meteorAppSettings);
-			appendKeyValue (vec, "ust_router_address",  ustRouterAddress);
-			appendKeyValue (vec, "ust_router_username", ustRouterUsername);
-			appendKeyValue (vec, "ust_router_password", ustRouterPassword);
 			appendKeyValue4(vec, "debugger",           debugger);
-			appendKeyValue4(vec, "analytics",          analytics);
 			appendKeyValue (vec, "api_key",            apiKey);
 
 			/*********************************/
@@ -634,21 +583,18 @@ public:
 			appendKeyValue2(vec, "max_preloader_idle_time", maxPreloaderIdleTime);
 			appendKeyValue3(vec, "max_out_of_band_work_instances", maxOutOfBandWorkInstances);
 		}
-		if ((fields & SPAWN_OPTIONS) || (fields & PER_GROUP_POOL_OPTIONS)) {
-			appendKeyValue (vec, "union_station_key",   unionStationKey);
-		}
 
 		/*********************************/
 	}
 
 	template<typename Stream>
 	void toXml(Stream &stream, const ResourceLocator &resourceLocator,
-		int fields = ALL_OPTIONS) const
+		const WrapperRegistry::Registry &wrapperRegistry, int fields = ALL_OPTIONS) const
 	{
 		vector<string> args;
 		unsigned int i;
 
-		toVector(args, resourceLocator, fields);
+		toVector(args, resourceLocator, wrapperRegistry, fields);
 		for (i = 0; i < args.size(); i += 2) {
 			stream << "<" << args[i] << ">";
 			stream << escapeForXml(args[i + 1]);
@@ -668,39 +614,47 @@ public:
 		}
 	}
 
-	string getStartCommand(const ResourceLocator &resourceLocator) const {
-		if (appType == P_STATIC_STRING("rack")) {
-			return ruby + "\t" + resourceLocator.getHelperScriptsDir() + "/rack-loader.rb";
-		} else if (appType == P_STATIC_STRING("wsgi")) {
-			return python + "\t" + resourceLocator.getHelperScriptsDir() + "/wsgi-loader.py";
-		} else if (appType == P_STATIC_STRING("node")) {
-			return nodejs + "\t" + resourceLocator.getHelperScriptsDir() + "/node-loader.js";
-		} else if (appType == P_STATIC_STRING("meteor")) {
-			return ruby + "\t" + resourceLocator.getHelperScriptsDir() + "/meteor-loader.rb";
+	string getStartCommand(const ResourceLocator &resourceLocator,
+		const WrapperRegistry::Registry &wrapperRegistry) const
+	{
+		const WrapperRegistry::Entry &entry = wrapperRegistry.lookup(appType);
+
+		string interpreter;
+		if (entry.language == P_STATIC_STRING("ruby")) {
+			interpreter = "/usr/local/bin/ruby26";
+		} else if (entry.language == P_STATIC_STRING("python")) {
+			interpreter = escapeShell(python);
+		} else if (entry.language == P_STATIC_STRING("nodejs")) {
+			interpreter = escapeShell(nodejs);
+		} else if (entry.language == P_STATIC_STRING("meteor")) {
+			interpreter = escapeShell(ruby);
 		} else {
-			return startCommand;
+			return appStartCommand;
 		}
+
+		return interpreter + " " + escapeShell(resourceLocator.getHelperScriptsDir()
+			+ "/" + entry.path);
 	}
 
-	StaticString getStartupFile() const {
+	StaticString getStartupFile(const WrapperRegistry::Registry &wrapperRegistry) const {
 		if (startupFile.empty()) {
-			const char *result = getAppTypeStartupFile(getAppType(appType));
-			if (result == NULL) {
-				return P_STATIC_STRING("");
+			const WrapperRegistry::Entry &entry = wrapperRegistry.lookup(appType);
+			if (entry.isNull() || entry.defaultStartupFiles.empty()) {
+				return StaticString();
 			} else {
-				return result;
+				return entry.defaultStartupFiles[0];
 			}
 		} else {
 			return startupFile;
 		}
 	}
 
-	StaticString getProcessTitle() const {
-		const char *result = getAppTypeProcessTitle(getAppType(appType));
-		if (result == NULL) {
-			return processTitle;
+	StaticString getProcessTitle(const WrapperRegistry::Registry &registry) const {
+		const WrapperRegistry::Entry &entry = registry.lookup(appType);
+		if (entry.isNull()) {
+			return entry.processTitle;
 		} else {
-			return result;
+			return StaticString();
 		}
 	}
 
@@ -717,4 +671,3 @@ public:
 } // namespace Passenger
 
 #endif /* _PASSENGER_APPLICATION_POOL2_OPTIONS_H_ */
-
